--
-- Copyright 2018 European Commission | CEF eDelivery
--
-- Licensed under the EUPL, Version 1.2 or – as soon they will be approved by the European Commission - subsequent versions of the EUPL (the Licence);
-- You may not use this work except in compliance with the Licence.
--
-- You may obtain a copy of the Licence attached in file: LICENCE-EUPL-v1.2.pdf
--
-- Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on an AS IS basis,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the Licence for the specific language governing permissions and limitations under the Licence.


/************************************************
* CREATE TABLES
************************************************/
CREATE TABLE SMP_CONFIGURATION  (
  PROPERTY VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	VALUE VARCHAR(4000) CHARACTER SET utf8 COLLATE utf8_bin,
	DESCRIPTION VARCHAR(4000) CHARACTER SET utf8 COLLATE utf8_bin,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	CONSTRAINT SMP_CONFIGURATION_PKEY PRIMARY KEY (PROPERTY)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_DOMAIN  (
  ID DECIMAL(38,0) NOT NULL,
	DOMAIN_CODE VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	SML_SUBDOMAIN VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	SML_SMP_ID VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin,
	SML_PARTC_IDENT_REGEXP VARCHAR(1024) CHARACTER SET utf8 COLLATE utf8_bin,
	SML_CLIENT_CERT_HEADER VARCHAR(4000) CHARACTER SET utf8 COLLATE utf8_bin,
	SML_CLIENT_KEY_ALIAS VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_bin,
	SIGNATURE_KEY_ALIAS VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_bin,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
  CONSTRAINT SMP_DOMAIN_PKEY PRIMARY KEY (ID)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;


CREATE UNIQUE INDEX SMP_DOMAIN_CODE_IDX ON SMP_DOMAIN (DOMAIN_CODE);
CREATE UNIQUE INDEX SMP_DOMAIN_CODE_IDX ON SMP_DOMAIN (SML_SUBDOMAIN);



CREATE TABLE SMP_SERVICE_GROUP (
  ID DECIMAL(38,0) NOT NULL,
	FK_DOMAIN_ID DECIMAL(38,0) NOT NULL,
	PARTICIPANT_IDENTIFIER VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	PARTICIPANT_SCHEME VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin,
	EXTENSION LONGTEXT,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	CONSTRAINT SMP_SERVICE_GROUP_PKEY PRIMARY KEY (ID),
	CONSTRAINT SMP_SG_DOMAIN_ID_FKEY FOREIGN KEY (FK_DOMAIN_ID) REFERENCES SMP_DOMAIN ( ID)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

/**speedup search */
CREATE INDEX SMP_SG_PART_ID_IDX ON SMP_SERVICE_GROUP (PARTICIPANT_IDENTIFIER);
CREATE INDEX SMP_SG_PART_SCH_IDX ON SMP_SERVICE_GROUP (PARTICIPANT_SCHEME);
CREATE UNIQUE INDEX SMP_SG_UNIQ_PARTC_IDX ON SMP_SERVICE_GROUP (FK_DOMAIN_ID, PARTICIPANT_SCHEME, PARTICIPANT_IDENTIFIER);


CREATE TABLE SMP_SERVICE_METADATA (
  ID DECIMAL(38,0) NOT NULL,
	FK_SG_ID DECIMAL(38,0) NOT NULL,
	DOCUMENT_IDENTIFIER VARCHAR(500) CHARACTER SET utf8 COLLATE utf8_bin,
	DOCUMENT_SCHEME VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_bin,
	XML_CONTENT LONGTEXT,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	CONSTRAINT SMP_SERVICE_METADATA_PKEY PRIMARY KEY (ID),
	CONSTRAINT MT_SG_ID_FKEY FOREIGN KEY (FK_SG_ID) REFERENCES SMP_SERVICE_GROUP (ID)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;


/**speedup search */
CREATE INDEX SMP_MD_DOC_ID_IDX ON SMP_SERVICE_METADATA (DOCUMENT_IDENTIFIER);
CREATE INDEX SMP_MD_DOC_SCH_IDX ON SMP_SERVICE_METADATA (DOCUMENT_SCHEME);
CREATE UNIQUE INDEX SMP_MD_UNIQ_DOC_IDX ON SMP_SERVICE_METADATA (FK_SG_ID, DOCUMENT_SCHEME, DOCUMENT_IDENTIFIER);

CREATE TABLE SMP_CERTIFICATE (
  ID DECIMAL(38,0) NOT NULL,
	CERTIFICATE_ID VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	VALID_FROM DATETIME NOT NULL,
	VALID_UNTIL DATETIME NOT NULL,
	PEM_ENCODING LONGTEXT,
	NEW_CERT_CHANGE_DATE DATETIME,
	NEW_CERT_ID DECIMAL(38,0),
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	CONSTRAINT SMP_CERTIFICATE_PKEY PRIMARY KEY (ID)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE UNIQUE INDEX SMP_CERTIFICATE_ID_IDX ON SMP_CERTIFICATE (CERTIFICATE_ID);

CREATE TABLE SMP_USER (
  ID DECIMAL(38,0) NOT NULL,
	USERNAME VARCHAR(256) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	PASSWORD VARCHAR(256) CHARACTER SET utf8 COLLATE utf8_bin,
	FK_CERTIFICATE_ID DECIMAL(38,0),
	ROLE VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	ACTIVE TINYINT DEFAULT 1 NOT NULL,
	CREATED_ON DATETIME NOT NULL,
	LAST_UPDATED_ON DATETIME NOT NULL,
	CONSTRAINT SMP_USER_PKEY PRIMARY KEY (ID),
	CONSTRAINT USER_CERTIFICATE_ID_FKEY FOREIGN KEY (ID) REFERENCES SMP_CERTIFICATE (ID),
	CONSTRAINT CHECK_ACTIVE_VALUE CHECK (ACTIVE = 0 OR ACTIVE = 1)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE UNIQUE INDEX SMP_USER_USERNAME_IDX ON SMP_USER (USERNAME);

CREATE TABLE SMP_OWNERSHIP (
	FK_USER_ID DOUBLE NOT NULL,
	FK_SG_ID DOUBLE NOT NULL,
	CONSTRAINT OWN_USER_SG_PKEY PRIMARY KEY (FK_USER_ID,FK_SG_ID )
);

/************************************************
* CREATE AUDIT TABLES
************************************************/
CREATE TABLE SMP_REV_INFO (
  ID DECIMAL(38, 0) NOT NULL,
  TIMESTAMP DECIMAL(38, 0),
  REVISION_DATE DATETIME(6),
  USERNAME VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin,
  CONSTRAINT PK_SMP_REV_INFO PRIMARY KEY (ID)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_CONFIGURATION_AUD  (
  PROPERTY VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	VALUE VARCHAR(4000) CHARACTER SET utf8 COLLATE utf8_bin,
	DESCRIPTION VARCHAR(4000) CHARACTER SET utf8 COLLATE utf8_bin,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	REV INTEGER NOT NULL,
  REVTYPE SMALLINT,
	CONSTRAINT SMP_CONFIGURATION_AUD_PKEY PRIMARY KEY (PROPERTY,REV)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_DOMAIN_AUD  (
  ID DECIMAL(38,0) NOT NULL,
	DOMAIN_CODE VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	SML_SUBDOMAIN VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin,
	SML_SMP_ID VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin,
	SML_PARTC_IDENT_REGEXP VARCHAR(1024) CHARACTER SET utf8 COLLATE utf8_bin,
	SML_CLIENT_CERT_HEADER VARCHAR(4000) CHARACTER SET utf8 COLLATE utf8_bin,
	SML_CLIENT_KEY_ALIAS VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_bin,
	SIGNATURE_KEY_ALIAS VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_bin,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	REV INTEGER NOT NULL,
  REVTYPE SMALLINT,
  CONSTRAINT SMP_DOMAIN_AUD_PKEY PRIMARY KEY (ID,REV)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_SERVICE_GROUP_AUD (
  ID DECIMAL(38,0) NOT NULL,
	FK_DOMAIN_ID DOUBLE NOT NULL,
	PARTICIPANT_IDENTIFIER VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	PARTICIPANT_SCHEME VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin,
	EXTENSION LONGTEXT,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	REV INTEGER NOT NULL,
  REVTYPE SMALLINT,
	CONSTRAINT SMP_SERVICE_GROUP_AUD_PKEY PRIMARY KEY (ID,REV)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_SERVICE_METADATA_AUD(
  ID DECIMAL(38,0) NOT NULL,
	FK_SG_ID DOUBLE NOT NULL,
	DOCUMENT_IDENTIFIER VARCHAR(500) CHARACTER SET utf8 COLLATE utf8_bin,
	DOCUMENT_SCHEME VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_bin,
	XML_CONTENT LONGTEXT,
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	REV INTEGER NOT NULL,
  REVTYPE SMALLINT,
	CONSTRAINT SMP_SERVICE_METADATA_AUD_PKEY PRIMARY KEY (ID,REV)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_CERTIFICATE_AUD (
  ID DECIMAL(38,0) NOT NULL,
	CERTIFICATE_ID VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	VALID_FROM DATETIME NOT NULL,
	VALID_UNTIL DATETIME NOT NULL,
	PEM_ENCODING LONGTEXT,
	NEW_CERT_CHANGE_DATE DATETIME,
	NEW_CERT_ID DECIMAL(38,0),
	CREATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	LAST_UPDATED_ON DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
	REV INTEGER NOT NULL,
  REVTYPE SMALLINT,
	CONSTRAINT SMP_CERTIFICATE_AUD_PKEY PRIMARY KEY (ID,REV)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_USER_AUD (
  ID DECIMAL(38,0) NOT NULL,
	USERNAME VARCHAR(256) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	PASSWORD VARCHAR(256) CHARACTER SET utf8 COLLATE utf8_bin,
	FK_CERTIFICATE_ID DOUBLE,
	ROLE VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	ACTIVE TINYINT DEFAULT 1 NOT NULL,
	CREATED_ON DATETIME NOT NULL,
	LAST_UPDATED_ON DATETIME NOT NULL,
	REV INTEGER NOT NULL,
  REVTYPE SMALLINT,
	CONSTRAINT SMP_USER_AUD_PKEY PRIMARY KEY (ID,REV)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8;

CREATE TABLE SMP_OWNERSHIP_AUD (
	FK_USER_ID DOUBLE NOT NULL,
	FK_SG_ID DOUBLE NOT NULL,
	REV INTEGER NOT NULL,
  REVTYPE SMALLINT,
	CONSTRAINT OWN_USER_SG_AUD_PKEY PRIMARY KEY (FK_USER_ID,FK_SG_ID,REV )
);


CREATE TABLE SMP_SEQUENCE_TABLE(
  sequence_name VARCHAR(64) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
  next_val BIGINT NOT NULL
);


INSERT INTO SMP_SEQUENCE_TABLE(sequence_name, next_val) values('SMP_CONFIGURATION_SEQ', 1);
INSERT INTO SMP_SEQUENCE_TABLE(sequence_name, next_val) values('SMP_DOMAIN_SEQ',1);
INSERT INTO SMP_SEQUENCE_TABLE(sequence_name, next_val) values('SMP_SERVICE_GROUP_SEQ', 1);
INSERT INTO SMP_SEQUENCE_TABLE(sequence_name, next_val) values('SMP_SERVICE_METADATA_SEQ',1);
INSERT INTO SMP_SEQUENCE_TABLE(sequence_name, next_val) values('SMP_CERTIFICATE_SEQ',1);
INSERT INTO SMP_SEQUENCE_TABLE(sequence_name, next_val) values('SMP_USER_SEQ',1);
INSERT INTO SMP_SEQUENCE_TABLE(sequence_name, next_val) values('SMP_REV_SEQ',1);




commit;
